<?php
/**
 * @copyright	Copyright (C) 2013 Omar Ramos. All rights reserved.
 * @license		GNU General Public License version 2 or later; see LICENSE.txt
 */

// no direct access
defined('_JEXEC') or die;

/**
 * Homepage Cache Plugin
 *
 * @package		Joomla.Plugin
 * @subpackage	System.cache
 */
class plgSystemHomepageCache extends JPlugin
{

	var $_cache = null;

	/**
	 * Constructor
	 *
	 * @access	protected
	 * @param	object	$subject The object to observe
	 * @param	array	$config  An array that holds the plugin configuration
	 * @since	1.0
	 */
	function __construct(& $subject, $config)
	{
		parent::__construct($subject, $config);

		//Set the language in the class
		$config = JFactory::getConfig();
		$options = array(
			'defaultgroup'	=> 'page',
			'browsercache'	=> $this->params->get('browsercache', false),
			'caching'		=> false,
		);

		$this->_cache = JCache::getInstance('page', $options);
	}

	/**
	* Converting the site URL to fit to the HTTP request
	*
	*/
	function onAfterInitialise()
	{
		global $_PROFILER;
		$app	= JFactory::getApplication();
		$user	= JFactory::getUser();

		if ($app->isAdmin()) {
			return;
		}

		$uri = JFactory::getURI()->toString();

		if ($this->isCacheable($uri))
		{
			if (count($app->getMessageQueue()))
			{
				return;
			}

			if ($user->get('guest') && $_SERVER['REQUEST_METHOD'] == 'GET')
			{
				$this->_cache->setCaching(true);
			}

			$data  = $this->_cache->get();

			if ($data !== false)
			{
				JResponse::setBody($data);

				if (JDEBUG)
				{
					$_PROFILER->mark('afterCache');
					$debugOutput =  implode('', $_PROFILER->getBuffer());
				}

				echo JResponse::toString($app->getCfg('gzip'));

				$app->close();
			}
		}
	}

	function onAfterRender()
	{
		$app = JFactory::getApplication();

		if ($app->isAdmin()) {
			return;
		}

		$uri = JFactory::getURI()->toString();

		if ($this->isCacheable($uri))
		{
			if (count($app->getMessageQueue()))
			{
				return;
			}

			$user = JFactory::getUser();
			if ($user->get('guest'))
			{
				//We need to check again here, because auto-login plugins have not been fired before the first aid check
				$this->_cache->store();
			}
		}
	}

	protected function isCacheable($uri = '')
	{
		if (empty($uri))
		{
			$uri = JFactory::getURI()->toString();
		}

		$uriSchemeHostPortPathParts = JFactory::getURI()->getScheme() . '://' . JFactory::getURI()->getHost() . JFactory::getURI()->getPort() . JFactory::getURI()->base(true);
		$relativeUriPathToCompare = str_replace($uriSchemeHostPortPathParts, '', $uri);

		$allowedRelativeUriPaths = array();
		if ($this->params->get('homepagecache', 1))
		{
			// Add relative paths that could be used for the homepage:
			$allowedRelativeUriPaths[] = '';
			$allowedRelativeUriPaths[] = '/';
			$allowedRelativeUriPaths[] = 'index.php';
		}

		if ($this->params->get('priority_one_cache_switch', 1))
		{
			// Add switchable on/off Priority One Cache Relative URLs:
			$priorityOneCache = $this->params->get('priority_one_cache', '');
			$priorityOneCache = JString::trim($priorityOneCache);

			if (!empty($priorityOneCache))
			{
				$priorityOneUris = explode("\r\n", $priorityOneCache);
				foreach($priorityOneUris as $priorityOneUri)
				{
					$priorityOneUri = JString::trim($priorityOneUri);
					$priorityOneUri = JString::ltrim($priorityOneUri, '/');
					$priorityOneUri = JString::rtrim($priorityOneUri, '/');
					$allowedRelativeUriPaths[] = $priorityOneUri;
				}
			}

		}

		if ($this->params->get('priority_two_cache_switch', 1))
		{
			// Add switchable on/off Priority Two Cache Relative URLs:
			$priorityTwoCache = $this->params->get('priority_two_cache', '');
			$priorityTwoCache = JString::trim($priorityTwoCache);

			if (!empty($priorityTwoCache))
			{
				$priorityTwoUris = explode("\r\n", $priorityTwoCache);
				foreach($priorityTwoUris as $priorityTwoUri)
				{
					$priorityTwoUri = JString::trim($priorityTwoUri);
					$priorityTwoUri = JString::ltrim($priorityTwoUri, '/');
					$priorityTwoUri = JString::rtrim($priorityTwoUri, '/');
					$allowedRelativeUriPaths[] = $priorityTwoUri;
				}
			}
		}

		if ($this->params->get('priority_three_cache_switch', 1))
		{
			// Add switchable on/off Priority Three Cache Relative URLs:
			$priorityThreeCache = $this->params->get('priority_three_cache', '');
			$priorityThreeCache = JString::trim($priorityThreeCache);

			if (!empty($priorityThreeCache))
			{
				$priorityThreeUris = explode("\r\n", $priorityThreeCache);
				foreach($priorityThreeUris as $priorityThreeUri)
				{
					$priorityThreeUri = JString::trim($priorityThreeUri);
					$priorityThreeUri = JString::ltrim($priorityThreeUri, '/');
					$priorityThreeUri = JString::rtrim($priorityThreeUri, '/');
					$allowedRelativeUriPaths[] = $priorityThreeUri;
				}
			}
		}

		$relativeUriPathToCompare = JString::ltrim($relativeUriPathToCompare, '/');
		$relativeUriPathToCompare = JString::rtrim($relativeUriPathToCompare, '/');
		$result = in_array($relativeUriPathToCompare, $allowedRelativeUriPaths);
		return $result;
	}
}
